from django.core.paginator import Paginator
from django.db.models import F
from xj_payment.models import PaymentStatus


class PaymentStatusService:
    @staticmethod
    def list(params):
        size = params.pop('size', 20)
        page = params.pop('page', 20)
        list_obj = PaymentStatus.objects.filter(**params).order_by('-id')
        count = list_obj.count()
        list_obj = list_obj.values(
            "id",
            "payment_status",
            "group",
            "description",
        )
        res_set = Paginator(list_obj, size).get_page(page)
        page_list = []
        if res_set:
            page_list = list(res_set.object_list)

        return {'total': count, 'page': page, 'size': size, "list": page_list}, None

    @staticmethod
    def add(params):
        try:
            PaymentStatus.objects.create(**params)
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)




from django.http import JsonResponse, HttpResponse
from django.views.decorators.http import require_http_methods
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from xj_enroll.service.enroll_services import EnrollServices
from xj_thread.services.thread_list_service import ThreadListService
from ..utils.custom_tool import request_params_wrapper, format_params_handle, flow_service_wrapper
from ..utils.user_wrapper import user_authentication_wrapper
from ..services.payment_service import PaymentService
from ..utils.model_handle import parse_data, util_response
from ..utils.utility_method import extract_values


class PaymentApis(APIView):

    # 支付列表
    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def list(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        params.setdefault("user_id", user_id)  # 用户ID
        params.setdefault("platform_id", platform_id)  # 平台
        # ================== 信息id列表反查询报名 start===============================
        thread_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["title", "subtitle", "access_level", "author"],
            is_remove_empty=True
        )
        if thread_params:
            thread_ids, err = ThreadListService.search_ids(search_prams=thread_params)
            if not err:
                request_params["thread_id_list"] = thread_ids

            # TODO 镖行的逻辑和其他逻辑不一样
            if isinstance(request_params.get("thread_id_list"), list) and len(
                    request_params["thread_id_list"]) == 0:
                request_params["thread_id_list"] = [0]
            print(request_params)
            if request_params.get("is_bx", None):
                enroll_list, err = EnrollServices.enroll_list({"thread_id_list": request_params["thread_id_list"]})
                if not err:
                    request_params["enroll_id_list"] = extract_values(enroll_list['list'], 'id')
                    if isinstance(request_params.get("enroll_id_list"), list) and len(
                            request_params["enroll_id_list"]) == 0:
                        request_params["enroll_id_list"] = [0]

                    request_params.pop("thread_id_list")
        # ================== 信息id列表反查询报名 end ===============================
        print(request_params)
        data, err_txt = PaymentService.list(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return util_response(data=data)

    # 支付总接口
    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def pay(self, *args, user_info, request_params, **kwargs, ):
        response = HttpResponse()
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        params.setdefault("user_id", user_id)  # 用户ID
        params.setdefault("platform_id", platform_id)  # 平台
        data, err_txt = PaymentService.pay(params)
        if err_txt:
            if isinstance(err_txt, dict) and err_txt.get("error"):
                content = util_response(err=int(err_txt['error']), msg=err_txt['msg'])
            else:
                content = util_response(err=47767, msg=err_txt)
        else:
            content = util_response(data=data)
        response.content = content
        return response

    @api_view(['GET'])
    @request_params_wrapper
    def ask_order_status(self, *args, request_params, **kwargs, ):
        params = request_params
        data, err_txt = PaymentService.ask_order_status(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return util_response(data=data)

    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def golden_touch(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        data, err_txt = PaymentService.golden_touch(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return util_response(data=data)

    @api_view(['GET'])
    @user_authentication_wrapper
    @request_params_wrapper
    def golden_touch(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        data, err_txt = PaymentService.golden_touch(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return util_response(data=data)

    @api_view(['POST'])
    @user_authentication_wrapper
    @request_params_wrapper
    def refund(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        data, err_txt = PaymentService.refund(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return util_response(data=data)


# 应用名称
from django.urls import re_path

from xj_payment.apis import alipay, transaction_inquiry, payment, payment_status
from xj_payment.apis.payment_apis import PaymentApis
from xj_payment.apis.payment_status_apis import PaymentStatusApis
from .apis import payment_alipay
from .apis import payment_wechat
from .apis import payment_unionpay
from .apis import payment

app_name = 'payment'

urlpatterns = [
    re_path(r'^get_pay_url/?$', payment_alipay.PaymentAlipay.get_pay_url),  # 获取支付宝支付链接
    re_path(r'^get_result/?$', payment_alipay.PaymentAlipay.pay_result),  # 支付宝处理完成后同步回调通知
    re_path(r'^update_order/?$',payment_alipay.PaymentAlipay.update_order),  # 支付宝处理完成后支付宝服务器异步回调通知
    re_path(r'^close/?$', alipay.Payment.close),  # 关闭订单

    re_path(r'^refund_inquiry/?$', transaction_inquiry.query.refund_inquiry),  # 支付宝退款查询
    re_path(r'^trade_query/?$', transaction_inquiry.query.trade_query),  # 支付宝下单查询

    re_path(r'^get_user_info/?$', payment_wechat.PaymentWechat.get_user_info),  # 获取用户标识接口
    re_path(r'^scan_pay/?$', payment_wechat.PaymentWechat.payment_scan_pay),  # 微信扫码支付
    re_path(r'^red_envelopes/?$', payment_wechat.PaymentWechat.payment_red_envelopes),  # 微信红包
    re_path(r'^wechat_callback/?$', payment_wechat.PaymentWechat.callback, name="微信回调接口"),  # 微信回调接口
    re_path(r'^wechat_callback_v3/?$', payment_wechat.PaymentWechat.callback_v3, name="微信回调接口（新）"),  # 微信回调接口
    re_path(r'^logic/?$', payment_wechat.PaymentWechat.logic),  # 支付逻辑处理模拟

    re_path(r'^unipay/?$', payment_unionpay.PaymentUnionPay.unipay),

    re_path(r'^refund/?$', payment.Payment.refund, name="退款总接口"),

    re_path(r'^pay/?$', PaymentApis.pay, name="支付总接接口"),  # 支付总接接口
    re_path(r'^list/?$', PaymentApis.list, name="支付列表"),
    re_path(r'^ask_order_status/?$', PaymentApis.ask_order_status),
    re_path(r'^golden_touch/?$', PaymentApis.golden_touch),

    re_path(r'^status_add/?$', PaymentStatusApis.add, name="支付状态添加"),  # 支付状态添加
    re_path(r'^status_list/?$', PaymentStatusApis.list, name="支付状态列表"),  # 支付状态列表
]



from django.http import JsonResponse
from rest_framework.views import APIView

from utils.custom_response import util_response
from utils.custom_tool import parse_data
from ..services.payment_satus_service import PaymentStatusService


class PaymentStatus(APIView):
    def get(self, request, *args, **kwargs):
        params = parse_data(request)
        data, err_txt = PaymentStatusService.get(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            'data': data
        })

    def post(self, request, *args, **kwargs):
        params = parse_data(request)
        data, err_txt = PaymentStatusService.post(params)
        if err_txt:
            return util_response(err=47767, msg=err_txt)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
        })



import os
import random
import string
import hashlib
import OpenSSL
import requests
import six
from datetime import datetime
import logging
from pathlib import Path
from wechatpy import WeChatPay
from wechatpy.pay.utils import get_external_ip
from wechatpayv3 import SignType, WeChatPay as WeChatPay3, WeChatPayType
from main.settings import BASE_DIR
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from logging import getLogger

# logger = getLogger('log')
# logger.info('---test---')

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
# APPID，应用ID或服务商模式下的sp_appid
app_id = main_config_dict.wechat_service_app_id or module_config_dict.wechat_service_app_id or ""
# 商户号（服务号）
mch_id = main_config_dict.wechat_service_mch_id or module_config_dict.wechat_service_mch_id or ""
# 小程序SECRET （服务号）
app_secret = main_config_dict.wechat_service_app_secret or module_config_dict.wechat_service_app_secret or ""
# 商户KEY（服务号）
merchant_key = main_config_dict.wechat_service_merchant_key or module_config_dict.wechat_service_merchant_key or ""
# 小程序ID（子商户）
sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""
# 小程序SECRET （子商户）
sub_app_secret = main_config_dict.wechat_merchant_app_secret or module_config_dict.wechat_merchant_app_secret or ""
# 微信支付商户号（直连模式）或服务商商户号（服务商模式，即sp_mchid)
sub_mch_id = main_config_dict.wechat_merchant_mch_id or module_config_dict.wechat_merchant_mch_id or ""
# API v3密钥， https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay3_2.shtml
apiv3_secret = main_config_dict.wechat_apiv3_secret or module_config_dict.wechat_apiv3_secret or ""
# 商户证书序列号
cert_serial_no = main_config_dict.wechat_cert_serial_no or module_config_dict.wechat_cert_serial_no or ""
# 微信支付平台证书缓存目录，减少证书下载调用次数
# 初始调试时可不设置，调试通过后再设置，示例值:'./cert'
cert_dir = main_config_dict.wechat_cert_dir or module_config_dict.wechat_cert_dir or ""
# 商品描述，商品简单描述
description = main_config_dict.wechat_body or module_config_dict.wechat_body or ""
# 商户名称
merchant_name = main_config_dict.merchant_name or module_config_dict.merchant_name or ""
# 标价金额，订单总金额，单位为分
total_fee = main_config_dict.wechat_total_fee or module_config_dict.wechat_total_fee or ""
# 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。回调地址，也可以在调用接口的时候覆盖
notify_url = main_config_dict.wechat_notify_url or module_config_dict.wechat_notify_url or ""
# 商户证书私钥路径
private_key_path = main_config_dict.wechat_merchant_private_key_file or module_config_dict.wechat_merchant_private_key_file or (
        str(BASE_DIR) + "/config/cert/apiclient_key.pem")
# 商户证书
cert_path = main_config_dict.wechat_merchant_cert_file or module_config_dict.wechat_merchant_cert_file or (
        str(BASE_DIR) + "/config/cert/apiclient_cert.pem")

# 商户证书私钥路径
son_private_key_path = main_config_dict.wechat_son_merchant_private_key_file or module_config_dict.wechat_son_merchant_private_key_file or (
        str(BASE_DIR) + "/config/cert/son/apiclient_key.pem")
# 商户证书
son_cert_path = main_config_dict.wechat_son_merchant_cert_file or module_config_dict.wechat_son_merchant_cert_file or (
        str(BASE_DIR) + "/config/cert/son/apiclient_cert.pem")

# 读取文件获取密钥
private_key = open(private_key_path, encoding="utf-8").read() if os.path.exists(private_key_path) else ""
# 日志记录器，记录web请求和回调细节
logging.basicConfig(filename=os.path.join(os.getcwd(), 'demo.log'), level=logging.DEBUG, filemode='a',
                    format='%(asctime)s - %(process)s - %(levelname)s: %(message)s')
LOGGER = logging.getLogger("demo")
# 接入模式:False=直连商户模式，True=服务商模式
PARTNER_MODE = True
# 代理设置，None或者{"https": "http://10.10.1.10:1080"}，详细格式参见https://docs.python-requests.org/zh_CN/latest/user/advanced.html
PROXY = None
url = "https://api.mch.weixin.qq.com/v3/pay/partner/transactions/jsapi"
logger = logging.getLogger(__name__)


def my_ali_pay():
    wechat = WeChatPay(
        appid=app_id,
        sub_appid=sub_appid,
        api_key=merchant_key,
        mch_id=mch_id,
        sub_mch_id=sub_mch_id,
        mch_key=apiv3_secret,
    )
    return wechat


def my_ali_pay_v3(wechatpay_type=WeChatPayType.JSAPI):
    wxpay = WeChatPay3(
        wechatpay_type=wechatpay_type,
        mchid=mch_id,
        private_key=private_key,
        cert_serial_no=cert_serial_no,
        apiv3_key=apiv3_secret,
        appid=app_id,
        notify_url=notify_url,
        cert_dir=cert_dir,
        logger=LOGGER,
        partner_mode=PARTNER_MODE,
        proxy=PROXY
    )
    return wxpay


def applets_red_envelopes():
    url = "https://api.mch.weixin.qq.com/mmpaymkttransfers/sendminiprogramhb"
    # path = '/mmpaymkttransfers/sendminiprogramhb'
    now = datetime.now()
    out_trade_no = '{0}{1}{2}'.format(sub_mch_id, now.strftime('%Y%m%d%H%M%S'), random.randint(1000, 10000))
    data = {
        'nonce_str': random_string(32),
        'mch_billno': out_trade_no,  # 商户订单号
        'mch_id': mch_id,  # 商户号
        'wxappid': app_id,  # 公众账号appid
        'send_name': merchant_name,  # 商户名称
        're_openid': 'oj8KR5AR0sV20j96VBcnWulf7bbs',  # 用户openid
        'total_amount': 1,  # 付款金额
        'total_num': 1,  # 红包发放总人数
        'wishing': '恭喜发财',  # 红包祝福语
        'act_name': '余额提现',  # 活动名称
        'remark': '余额提现红包，请查收',  # 备注
        'notify_way': "MINI_PROGRAM_JSAPI",  # 通知用户形式
    }
    sign = calculate_signature(data, merchant_key)
    print(data)
    data['sign'] = sign
    body = dict_to_xml(data)
    amount = 1 / 100
    if amount >= 200:
        data['scene_id'] = 'PRODUCT_3'
    wx_key = os.path.join(private_key_path)
    wx_cert = os.path.join(cert_path)
    response = requests.post(url=url, data=body.encode('utf-8'),
                             cert=(wx_cert, wx_key))

    # response = requests.request("POST", url, headers=headers, data=body.encode("utf-8"))
    # response = requests.request("POST", url, data=body.encode('utf-8'), cert=(wx_cert, wx_key), verify=True)
    return response.text
    # return None, None


def dict_to_xml(d, sign=None):
    xml = ['<xml>\n']
    for k in sorted(d):
        # use sorted to avoid test error on Py3k
        v = d[k]
        if isinstance(v, six.integer_types) or (isinstance(v, six.string_types) and v.isdigit()):
            xml.append('<{0}>{1}</{0}>\n'.format(to_text(k), to_text(v)))
        else:
            xml.append(
                '<{0}><![CDATA[{1}]]></{0}>\n'.format(to_text(k), to_text(v))
            )
    if sign:
        xml.append('<sign><![CDATA[{0}]]></sign>\n</xml>'.format(to_text(sign)))
    else:
        xml.append('</xml>')
    return ''.join(xml)


def format_url(params, api_key=None):
    data = [to_binary('{0}={1}'.format(k, params[k])) for k in sorted(params) if params[k]]
    if api_key:
        data.append(to_binary('key={0}'.format(api_key)))
    return b"&".join(data)


def calculate_signature(params, api_key):
    url = format_url(params, api_key)
    logger.debug("Calculate Signature URL: %s", url)
    return to_text(hashlib.md5(url).hexdigest().upper())


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return to_text(value).encode(encoding)


def random_string(length=16):
    rule = string.ascii_letters + string.digits
    rand_list = random.sample(rule, length)
    return ''.join(rand_list)


def to_text(value, encoding='utf-8'):
    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)



# -*- coding: utf-8 -*-
import json
import os

import redis


# 项目环境配置信息
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
CONFIG_INFO = {}
try:
    CONFIG_FILE = BASE_DIR + '/.env'
    CONFIG_INFO = open(CONFIG_FILE).read()
    CONFIG_INFO = json.loads(CONFIG_INFO)
except Exception as E:
    print(E)
    print("没有找到项目env文件...")


def get_domain(request):
    """获取域名"""
    # 这里的域名应该为当前项目的域名，使用request.scheme 获取协议类型，使用request.META['HTTP_HOST'] 获取当前项目的域名
    domain = request.scheme + "://" + request.META['HTTP_HOST']

    return domain


class Redis(object):
    def __init__(self):
        self.r = None
        self.host = CONFIG_INFO.get("REDIS_HOST")
        self.port = CONFIG_INFO.get("REDIS_PORT")
        self.password = CONFIG_INFO.get("REDIS_PASSWORD")
        self.db = CONFIG_INFO.get("REDIS_DB")

    def connection(self):
        """连接 redis"""
        # decode_responses 将获取到的字节码转为字符串；
        # socket_connect_timeout 设置连接超时时间为 1s；
        # socket_time 设置连接建立后的读写操作超时为 1s；
        self.r = redis.StrictRedis(host=self.host,
                                   port=self.port,
                                   password=self.password,
                                   db=self.db,
                                   decode_responses=True,
                                   socket_connect_timeout=1,
                                   socket_timeout=1)
        return self.r



import random
from urllib import parse

from django.http import JsonResponse, HttpResponse
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt
from rest_framework.views import APIView

from xj_payment.common import get_domain
# from xj_payment.services.payment_service import PaymentService
from xj_payment.utils.utils_v1 import my_ali_pay, is_app_pay


# redis连接对象

# redis = Redis().connection()


class Payment(APIView):

    def get_pay_url(self):
        """
        获取支付连接
        :param request:
        :return:
        """
        if self.method == "GET":
            order_id = self.GET.get('order_id', 0)  # 前端传回的订单id
            money = self.GET.get('price')  # 前端传回的金额数据
            out_trade_no = self.GET.get('out_trade_no')  # 前端传回的订单号

            # if not all([order_id, money]):
            #     return JsonResponse(dict(message="参数错误"))

            # 此处可增加根据订单id查询判断该订单是否存在相关业务逻辑
            # 组织订单编号：当前时间字符串 + 6位随机数 ---> 20200808154711123456
            out_trade_no = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(map(str, random.sample(range(0, 9), 6)))
            # 生成支付宝支付链接地址
            domain_name = get_domain(self)
            notify_url = domain_name + '/api/payment/update_order/'
            alipay = my_ali_pay(notify_url)
            order_string = alipay.api_alipay_trade_page_pay(
                out_trade_no=out_trade_no,  # 订单编号
                total_amount=str(money),  # 交易金额(单位: 元 保留俩位小数)   这里一般是从前端传过来的数据
                subject=f"紫薇系统-{out_trade_no}",  # 商品名称或产品名称
                return_url=domain_name + "/api/payment/get_result/",  # 支付成功后跳转的页面，App支付此参数无效，集成支付宝SDK自带跳转
            )
            gatway = Config.getIns().get("xj_payment", "GATEWAY", "https://openapi.alipaydev.com/gateway.do")
            # print(123)
            # 拼接支付链接，注意：App支付不需要返回支付宝网关
            ali_pay_url = order_string if is_app_pay(order_string) else gatway + "?" + order_string

            # return JsonResponse(dict(ali_pay_url=ali_pay_url))

            res = {
                'err': 0,
                'msg': 'OK',
                'data': {
                    'ali_pay_url': ali_pay_url,
                },
            }
        # return JsonResponse(dict(ali_pay_url=""))
        return JsonResponse(res)

    @csrf_exempt
    def pay_result(self):
        """
        支付完成后，前端同步通知回调
        :param request:
        :return: 根据业务需求自定义返回信息
        """
        if self.method == "GET":
            data = self.GET.dict()
            # 同步验签data参数转换字典后示例如下：
            """
            {
                'charset': 'utf-8', 
                'out_trade_no': '20200808154711123456', 
                'method': 'alipay.trade.page.pay.return', 
                'total_amount': '0.01', 
                'sign': 'OBCdRRpUHtjAR15v9s26cU1juP+ub0COKRe3hJg2kCsMZIhCT3Kx......meYt0G2Kir/Ld77gp+OFLza2G5HrIrA==', 
                'trade_no': '2020080622001460481437011111', 
                'auth_app_id': '2016101000655892', 
                'version': '1.0', 
                'app_id': '2016101000655892', 
                'sign_type': 'RSA2', 
                'seller_id': '2078131328364326', 
                'timestamp': '2020-08-06 14:47:25'
            }
            """
            ali_pay = my_ali_pay()
            sign = data.pop('sign', None)
            success = ali_pay.verify(data, sign)
            # print("同步回调验签状态: ", success)
            if success:
                # 此处写支付验签成功的相关业务逻辑
                return JsonResponse(dict(message="支付成功"))

            return JsonResponse(dict(message="支付失败"))

        return JsonResponse(dict(message="支付失败"))

    # @csrf_exempt
    # @atomic()
    def update_order(self):
        """
        支付成功后，支付宝服务器异步通知回调（用于修改订单状态）
        :param request:
        :return: success or fail
        """
        if self.method == "POST":
            print("异步通知验证开始")
            body_str = self.body.decode('utf-8')
            data = parse.parse_qs(body_str)
            # data = parse.parse_qs(parse.unquote(body))  # 回传的url中如果发现被编码，这里可以用unquote解码再转换成字典
            # 异步通知data参数转换字典后示例如下：
            payment = None
            refund = None
            close = None
            try:
                gmt_payment = data.get('gmt_payment', [None])
                payment = gmt_payment[0]
                gmt_refund = data.get('gmt_refund', [None])
                refund = gmt_refund[0]
                gmt_close = data.get('gmt_close', [None])
                close = gmt_close[0]
            except KeyError as e:
                print(data)

            callback_data = {
                'app_id': data['app_id'][0],
                'order_no': data['out_trade_no'][0],
                'transact_no': data['trade_no'][0],
                'subject': data['subject'][0],
                'total_amount': data['total_amount'][0],
                'buyer_pay_amount': data['buyer_pay_amount'][0],
                'point_amount': data['point_amount'][0],
                'invoice_amount': data['invoice_amount'][0],
                'pay_mode': 1,
                'order_time': data['gmt_create'][0],
                'payment_time': payment,
                'refunt_time': refund,
                'close_time': close,
                # 'voucher_detail': data['vocher_detail_list']
            }
            if data['trade_status'][0] == 'WAIT_BUYER_PAY':
                callback_data['payment_status_id'] = 1
            elif data['trade_status'][0] == 'TRADE_CLOSED':
                callback_data['payment_status_id'] = 2
            elif data['trade_status'][0] == 'TRADE_SUCCESS':
                callback_data['payment_status_id'] = 3
            elif data['trade_status'][0] == 'TRADE_FINISHED':
                callback_data['payment_status_id'] = 4
            else:
                callback_data['payment_status_id'] = ''
            """
            {
                'auth_app_id': '2016101000655892', 
                'version': '1.0', 
                'charset': 'utf-8', 
                'subject': '产品名称-20200808154711123456', 
                'trade_status': 'TRADE_SUCCESS', 
                'app_id': '2016101000655892', 
                'total_amount': '0.01', 
                'buyer_pay_amount': '0.01', 
                'receipt_amount': '0.01', 
                'point_amount': '0.00', 
                'invoice_amount': '0.01', 
                'trade_no': '2020080622001460481436956490', 
                'sign_type': 'RSA2', 
                'buyer_id': '2083402614260483', 
                'notify_time': '2020-08-06 12:38:06', 
                'notify_id': '2020080600125143806060481455916209', 
                'notify_type': 'trade_status_sync', 
                'fund_bill_list': '[{"amount":"0.01","fundChannel":"PCREDIT"}]', 
                'gmt_create': '2020-08-06 12:38:02', 
                'gmt_payment': '2020-08-06 12:38:06', 
                'seller_id': '2078131328364326', 
                'out_trade_no': '20200808154711123456', 
                'sign': 'YNeo9DqaKZzCLwN+7zYMCeYn6+pmo5fxCv/KtCWa8zBzNNKowRc23......iU30qCPFSzq/t4UtJ4TwA5/pfHo9cNlbKQA=='
            }
            """

            data = {k: v[0] for k, v in data.items()}

            ali_pay = my_ali_pay()
            sign = data.pop('sign', None)
            success = ali_pay.verify(data, sign)  # 返回验签结果, True/False
            print("异步通知验证状态: ", success)
            if success:
                # 此处写支付验签成功修改订单状态相关业务逻辑
                PaymentService.create(self, callback_data)
                return HttpResponse('success')  # 返回success给支付宝服务器, 若支付宝收不到success字符会重复发送通知
            return HttpResponse('fail')

        return HttpResponse('fail')

    @csrf_exempt
    def refund(self):
        """
         退款
        :param request:
        :return: success or fail
         """

        if self.method == "POST":
            # 根据当前用户的配置，生成URL，并跳转。
            # print(self)
            money = (self.POST.get('price'))
            out_trade_no = self.POST.get('out_trade_no')

            # 实例化支付类
            alipay = my_ali_pay()
            # 调用退款方法
            domain_name = get_domain(self)
            notify_url = domain_name + '/api/payment/update_order/'
            order_string = alipay.api_alipay_trade_refund(
                # 订单号，一定要注意，这是支付成功后返回的唯一订单号
                out_trade_no=str(out_trade_no),
                # 退款金额，注意精确到分，不要超过订单支付总金额
                refund_amount=money,
                # 回调网址
                notify_url=notify_url
            )
            # 通知data参数转换字典后示例如下：
            """
            {
                "code": "10000",
                "msg": "Success",
                "buyer_logon_id": "jvg***@sandbox.com",
                "buyer_user_id": "2088622955554434",
                "fund_change": "Y",
                "gmt_refund_pay": "2022-08-10 14:01:02",
                "out_trade_no": "20220810054053350427",
                "refund_fee": "200.00",
                "send_back_fee": "0.00",
                "trade_no": "2022081022001454430501929100"
            }
            """
            # print(order_string)
            return JsonResponse(order_string)

    @csrf_exempt
    def close(self):
        if self.method == "POST":
            out_trade_no = self.POST.get('out_trade_no')
            trade_no = self.POST.get('trade_no')
            alipay = my_ali_pay()
            order_string = alipay.api_alipay_trade_close(
                out_trade_no=str(out_trade_no),
                trade_no=str(trade_no),
            )
            return JsonResponse(order_string)



from django.http import JsonResponse
from rest_framework.views import APIView

from xj_payment.utils.utils_v1 import my_ali_pay


class query(APIView):

    def refund_inquiry(self):
        """
         退款查询接口
         :param request:
          :return:
          """
        if self.method == "GET":
            out_trade_no = self.GET.get('out_trade_no')
            trade_no = self.GET.get('trade_no')
            alipay = my_ali_pay()
            order_string = alipay.api_alipay_trade_fastpay_refund_query(
                trade_no=str(trade_no),
                out_request_no=str(out_trade_no),
            )
            return JsonResponse(order_string)

    def trade_query(self):
        """
        支付查询接口
        :param request:
         :return:
          """
        if self.method == "GET":
            out_trade_no = self.GET.get('out_trade_no')
            trade_no = self.GET.get('trade_no')
            alipay = my_ali_pay()
            order_string = alipay.api_alipay_trade_query(
                trade_no=str(trade_no),
                out_trade_no=str(out_trade_no),
            )
            return JsonResponse(order_string)



# -*- coding: utf-8 -*-

import json
import time
import uuid
from base64 import b64decode, b64encode

from cryptography.exceptions import InvalidSignature, InvalidTag
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric.padding import MGF1, OAEP, PKCS1v15
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from cryptography.hazmat.primitives.hashes import SHA1, SHA256, SM3, Hash
from cryptography.hazmat.primitives.hmac import HMAC
from cryptography.hazmat.primitives.serialization import load_pem_private_key
from cryptography.x509 import load_pem_x509_certificate


def build_authorization(path,
                        method,
                        mchid,
                        serial_no,
                        private_key,
                        data=None,
                        nonce_str=None):
    timeStamp = str(int(time.time()))
    nonce_str = nonce_str or ''.join(str(uuid.uuid4()).split('-')).upper()
    body = data if isinstance(data, str) else json.dumps(data) if data else ''
    sign_str = '%s\n%s\n%s\n%s\n%s\n' % (method, path, timeStamp, nonce_str, body)
    signature = rsa_sign(private_key=private_key, sign_str=sign_str)
    authorization = 'WECHATPAY2-SHA256-RSA2048 mchid="%s",nonce_str="%s",signature="%s",timestamp="%s",serial_no="%s"' % (mchid, nonce_str, signature, timeStamp, serial_no)
    return authorization


def rsa_sign(private_key, sign_str):
    message = sign_str.encode('UTF-8')
    signature = private_key.sign(data=message, padding=PKCS1v15(), algorithm=SHA256())
    sign = b64encode(signature).decode('UTF-8').replace('\n', '')
    return sign


def aes_decrypt(nonce, ciphertext, associated_data, apiv3_key):
    key_bytes = apiv3_key.encode('UTF-8')
    nonce_bytes = nonce.encode('UTF-8')
    associated_data_bytes = associated_data.encode('UTF-8')
    data = b64decode(ciphertext)
    aesgcm = AESGCM(key=key_bytes)
    try:
        result = aesgcm.decrypt(nonce=nonce_bytes, data=data, associated_data=associated_data_bytes).decode('UTF-8')
    except InvalidTag:
        result = None
    return result


def format_private_key(private_key_str):
    pem_start = '-----BEGIN PRIVATE KEY-----\n'
    pem_end = '\n-----END PRIVATE KEY-----'
    if not private_key_str.startswith(pem_start):
        private_key_str = pem_start + private_key_str
    if not private_key_str.endswith(pem_end):
        private_key_str = private_key_str + pem_end
    return private_key_str


def load_certificate(certificate_str):
    try:
        return load_pem_x509_certificate(data=certificate_str.encode('UTF-8'), backend=default_backend())
    except:
        return None


def load_private_key(private_key_str):
    try:
        return load_pem_private_key(data=format_private_key(private_key_str).encode('UTF-8'), password=None, backend=default_backend())
    except:
        raise Exception('failed to load private key.')


def rsa_verify(timestamp, nonce, body, signature, certificate):
    sign_str = '%s\n%s\n%s\n' % (timestamp, nonce, body)
    public_key = certificate.public_key()
    message = sign_str.encode('UTF-8')
    signature = b64decode(signature)
    try:
        public_key.verify(signature, message, PKCS1v15(), SHA256())
    except InvalidSignature:
        return False
    return True


def rsa_encrypt(text, certificate):
    data = text.encode('UTF-8')
    public_key = certificate.public_key()
    cipherbyte = public_key.encrypt(
        plaintext=data,
        padding=OAEP(mgf=MGF1(algorithm=SHA1()), algorithm=SHA1(), label=None)
    )
    return b64encode(cipherbyte).decode('UTF-8')


def rsa_decrypt(ciphertext, private_key):
    data = private_key.decrypt(
        ciphertext=b64decode(ciphertext),
        padding=OAEP(mgf=MGF1(algorithm=SHA1()), algorithm=SHA1(), label=None)
    )
    result = data.decode('UTF-8')
    return result


def hmac_sign(key, sign_str):
    hmac = HMAC(key.encode('UTF-8'), SHA256())
    hmac.update(sign_str.encode('UTF-8'))
    sign = hmac.finalize().hex().upper()
    return sign


def sha256(data):
    hash = Hash(SHA256())
    hash.update(data)
    return hash.finalize().hex()


def sm3(data):
    hash = Hash(SM3())
    hash.update(data)
    return hash.finalize().hex()
